<?php

namespace Drupal\paytr_payment\Access;

use Drupal\commerce_order\Entity\Order;
use Drupal\Core\Routing\Access\AccessInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Access\AccessResult;
use Psr\Log\LoggerInterface;

/**
 * Checks access for the payment callback from QuickPay.
 */
class PaytrPaymentCallbackAccessCheck implements AccessInterface {

  /**
   * @var LoggerInterface
   */
  protected $logger;

  /**
   * PaytrPaymentCallbackAccessCheck constructor.
   * @param LoggerInterface $logger
   */
  public function __construct(LoggerInterface $logger) {
    $this->logger = $logger;
  }

  /**
   * @param Request $request
   * @return AccessResult
   */
  public function access(Request $request): AccessResult
  {
    if(Order::load($this->resolveOrderId($request)) !== null)
    {
      return AccessResult::allowed();
    }
    $this->logger->error("Böyle bir sipariş bulunamadı.");
    return AccessResult::forbidden();
  }

  /**
   * @param $request
   * @return int
   */
  private function resolveOrderId($request): int
  {
    $merchant_oid = $request->get('merchant_oid');
    $merchant_oid = explode('DR', $merchant_oid);
    $merchant_oid = str_replace('SP', '', $merchant_oid[0]);
    return (int) $merchant_oid;
  }
}
