<?php

namespace Drupal\paytr_payment\Controller;

use Drupal;
use Drupal\commerce_order\Entity\Order;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\paytr_payment\Helpers\PaytrHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class CallbackController
 * @package Drupal\paytr_payment\Controller
 */
class CallbackController extends ControllerBase {

  /**
   * @var EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): CallbackController
  {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * @param Request $request
   * @return Response
   * @throws InvalidPluginDefinitionException
   * @throws PluginNotFoundException
   * @throws EntityStorageException
   */
  public function callback(Request $request): Response
  {
    $payment = $this->entityTypeManager->getStorage('commerce_payment')->loadByRemoteId($this->resolveOrderId($request));
    $state   =
      $request->get('status') === 'success' &&
      $this->makeHash($request, $payment) === $request->get('hash') ? 'completed' : 'canceled';
    $order = Order::load($this->resolveOrderId($request));
    if($state==='canceled')
    {
      $order->set('state', $state);
      $order->save();
      return new Response('OK');
    }
    $payment->set('state', $state);
    $payment->set('remote_id', $request->get('merchant_oid'));
    $payment->save();
    $order->set('state', $state);
    $order->save();
    $logger = Drupal::logger('paytr_payment');
    $logger->info('Saving Payment information. Transaction reference: '.$request->get('merchant_oid'));
    return new Response('OK');
  }

  /**
   * @param $request
   * @return int
   */
  private function resolveOrderId($request): int
  {
    $merchant_oid = $request->get('merchant_oid');
    $merchant_oid = explode('DR', $merchant_oid);
    $merchant_oid = str_replace('SP', '', $merchant_oid[0]);
    return (int) $merchant_oid;
  }

  /**
   * @param $request
   * @param $payment
   * @return string
   */
  private function makeHash($request, $payment): string
  {
    $paytrHelper  = new PaytrHelper($payment);
    return base64_encode( hash_hmac('sha256', $request->get('merchant_oid').$paytrHelper->getMerchantSalt().$request->get('status').$request->get('total_amount'), $paytrHelper->getMerchantKey(), true) );
  }

}
